@@ -81,11 +81,6 @@ minetest.register_craftitem("default:clay_brick", {
 	inventory_image = "default_clay_brick.png",
 })
 
-minetest.register_craftitem("default:scorched_stuff", {
-	description = "Scorched Stuff",
-	inventory_image = "default_scorched_stuff.png",
-})
-
 minetest.register_craftitem("default:obsidian_shard", {
 	description = "Obsidian Shard",
 	inventory_image = "default_obsidian_shard.png",
