@@ -27,10 +27,14 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "util/numeric.h"
 #include <ICameraSceneNode.h>
 
+#include "client.h"
+
 class LocalPlayer;
 struct MapDrawControl;
 class IGameDef;
 
+enum CameraMode {CAMERA_MODE_FIRST, CAMERA_MODE_THIRD, CAMERA_MODE_THIRD_FRONT};
+
 /*
 	Client camera class, manages the player and camera scene nodes, the viewing distance
 	and performs view bobbing etc. It also displays the wielded tool in front of the
@@ -79,6 +83,12 @@ public:
 	{
 		return m_camera_direction;
 	}
+	
+	// Get the camera offset
+	inline v3s16 getOffset() const
+	{
+		return m_camera_offset;
+	}
 
 	// Horizontal field of view
 	inline f32 getFovX() const
@@ -107,7 +117,7 @@ public:
 	// Update the camera from the local player's position.
 	// busytime is used to adjust the viewing range.
 	void update(LocalPlayer* player, f32 frametime, f32 busytime,
-			v2u32 screensize, f32 tool_reload_ratio);
+			f32 tool_reload_ratio, ClientEnvironment &c_env);
 
 	// Render distance feedback loop
 	void updateViewingRange(f32 frametime_in, f32 busytime_in);
@@ -122,11 +132,26 @@ public:
 	// Draw the wielded tool.
 	// This has to happen *after* the main scene is drawn.
 	// Warning: This clears the Z buffer.
-	void drawWieldedTool();
+	void drawWieldedTool(irr::core::matrix4* translation=NULL);
+
+	// Toggle the current camera mode
+	void toggleCameraMode() {
+		if (m_camera_mode == CAMERA_MODE_FIRST)
+			m_camera_mode = CAMERA_MODE_THIRD;
+		else if (m_camera_mode == CAMERA_MODE_THIRD)
+			m_camera_mode = CAMERA_MODE_THIRD_FRONT;
+		else
+			m_camera_mode = CAMERA_MODE_FIRST;
+	}
+
+	//read the current camera mode
+	inline CameraMode getCameraMode()
+	{
+		return m_camera_mode;
+	}
 
 private:
-	// Scene manager and nodes
-	scene::ISceneManager* m_smgr;
+	// Nodes
 	scene::ISceneNode* m_playernode;
 	scene::ISceneNode* m_headnode;
 	scene::ICameraSceneNode* m_cameranode;
@@ -144,6 +169,8 @@ private:
 	v3f m_camera_position;
 	// Absolute camera direction
 	v3f m_camera_direction;
+	// Camera offset
+	v3s16 m_camera_offset;
 
 	// Field of view and aspect ratio stuff
 	f32 m_aspect;
@@ -184,6 +211,8 @@ private:
 	scene::IMesh *m_wield_mesh_next;
 	u16 m_previous_playeritem;
 	std::string m_previous_itemname;
+
+	CameraMode m_camera_mode;
 };
 
 #endif
