@@ -29,6 +29,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "mapblock.h"
 #include "profiler.h"
 #include "settings.h"
+#include "camera.h" // CameraModes
 #include "util/mathconstants.h"
 #include <algorithm>
 
@@ -57,7 +58,7 @@ ClientMap::ClientMap(
 ClientMap::~ClientMap()
 {
 	/*JMutexAutoLock lock(mesh_mutex);
-	
+
 	if(mesh != NULL)
 	{
 		mesh->drop();
@@ -75,15 +76,15 @@ MapSector * ClientMap::emergeSector(v2s16 p2d)
 	catch(InvalidPositionException &e)
 	{
 	}
-	
+
 	// Create a sector
 	ClientMapSector *sector = new ClientMapSector(this, p2d, m_gamedef);
-	
+
 	{
 		//JMutexAutoLock lock(m_sector_mutex); // Bulk comment-out
 		m_sectors[p2d] = sector;
 	}
-	
+
 	return sector;
 }
 
@@ -94,7 +95,7 @@ void ClientMap::deSerializeSector(v2s16 p2d, std::istream &is)
 	ClientMapSector *sector = NULL;
 
 	//JMutexAutoLock lock(m_sector_mutex); // Bulk comment-out
-	
+
 	core::map<v2s16, MapSector*>::Node *n = m_sectors.find(p2d);
 
 	if(n != NULL)
@@ -175,6 +176,7 @@ void ClientMap::updateDrawList(video::IVideoDriver* driver)
 	v3f camera_position = m_camera_position;
 	v3f camera_direction = m_camera_direction;
 	f32 camera_fov = m_camera_fov;
+	v3s16 camera_offset = m_camera_offset;
 	m_camera_mutex.Unlock();
 
 	// Use a higher fov to accomodate faster camera movements.
@@ -196,7 +198,7 @@ void ClientMap::updateDrawList(video::IVideoDriver* driver)
 			p_nodes_max.X / MAP_BLOCKSIZE + 1,
 			p_nodes_max.Y / MAP_BLOCKSIZE + 1,
 			p_nodes_max.Z / MAP_BLOCKSIZE + 1);
-	
+
 	// Number of blocks in rendering range
 	u32 blocks_in_range = 0;
 	// Number of blocks occlusion culled
@@ -221,7 +223,7 @@ void ClientMap::updateDrawList(video::IVideoDriver* driver)
 	{
 		MapSector *sector = si->second;
 		v2s16 sp = sector->getPos();
-		
+
 		if(m_control.range_all == false)
 		{
 			if(sp.X < p_blocks_min.X
@@ -233,13 +235,13 @@ void ClientMap::updateDrawList(video::IVideoDriver* driver)
 
 		std::list< MapBlock * > sectorblocks;
 		sector->getBlocks(sectorblocks);
-		
+
 		/*
 			Loop through blocks in sector
 		*/
 
 		u32 sector_blocks_drawn = 0;
-		
+
 		std::list< MapBlock * >::iterator i;
 		for(i=sectorblocks.begin(); i!=sectorblocks.end(); i++)
 		{
@@ -249,7 +251,10 @@ void ClientMap::updateDrawList(video::IVideoDriver* driver)
 				Compare block position to camera position, skip
 				if not seen on display
 			*/
-			
+
+			if (block->mesh != NULL)
+				block->mesh->updateCameraOffset(m_camera_offset);
+
 			float range = 100000 * BS;
 			if(m_control.range_all == false)
 				range = m_control.wanted_range * BS;
@@ -268,7 +273,7 @@ void ClientMap::updateDrawList(video::IVideoDriver* driver)
 				continue;*/
 
 			blocks_in_range++;
-			
+
 			/*
 				Ignore if mesh doesn't exist
 			*/
@@ -329,7 +334,7 @@ void ClientMap::updateDrawList(video::IVideoDriver* driver)
 				blocks_occlusion_culled++;
 				continue;
 			}
-			
+
 			// This block is in range. Reset usage timer.
 			block->resetUsageTimer();
 
@@ -378,18 +383,25 @@ struct MeshBufList
 struct MeshBufListList
 {
 	std::list<MeshBufList> lists;
-	
+
 	void clear()
 	{
 		lists.clear();
 	}
-	
+
 	void add(scene::IMeshBuffer *buf)
 	{
 		for(std::list<MeshBufList>::iterator i = lists.begin();
 				i != lists.end(); ++i){
 			MeshBufList &l = *i;
-			if(l.m == buf->getMaterial()){
+			video::SMaterial &m = buf->getMaterial();
+
+			// comparing a full material is quite expensive so we don't do it if
+			// not even first texture is equal
+			if (l.m.TextureLayer[0].Texture != m.TextureLayer[0].Texture)
+				continue;
+
+			if (l.m == m) {
 				l.bufs.push_back(buf);
 				return;
 			}
@@ -406,7 +418,7 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 	DSTACK(__FUNCTION_NAME);
 
 	bool is_transparent_pass = pass == scene::ESNRP_TRANSPARENT;
-	
+
 	std::string prefix;
 	if(pass == scene::ESNRP_SOLID)
 		prefix = "CM: solid: ";
@@ -427,7 +439,7 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 
 	/*
 		Get time for measuring timeout.
-		
+
 		Measuring time is very useful for long delays when the
 		machine is swapping a lot.
 	*/
@@ -451,7 +463,7 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 	*/
 
 	v3s16 cam_pos_nodes = floatToInt(camera_position, BS);
-	
+
 	v3s16 box_nodes_d = m_control.wanted_range * v3s16(1,1,1);
 
 	v3s16 p_nodes_min = cam_pos_nodes - box_nodes_d;
@@ -467,14 +479,14 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 			p_nodes_max.X / MAP_BLOCKSIZE + 1,
 			p_nodes_max.Y / MAP_BLOCKSIZE + 1,
 			p_nodes_max.Z / MAP_BLOCKSIZE + 1);
-	
+
 	u32 vertex_count = 0;
 	u32 meshbuffer_count = 0;
-	
+
 	// For limiting number of mesh animations per frame
 	u32 mesh_animate_count = 0;
 	u32 mesh_animate_count_far = 0;
-	
+
 	// Blocks that were drawn and had a mesh
 	u32 blocks_drawn = 0;
 	// Blocks which had a corresponding meshbuffer for this pass
@@ -500,7 +512,7 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 		// If the mesh of the block happened to get deleted, ignore it
 		if(block->mesh == NULL)
 			continue;
-		
+
 		float d = 0.0;
 		if(isBlockInSight(block->getPos(), camera_position,
 				camera_direction, camera_fov,
@@ -572,9 +584,9 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 			}
 		}
 	}
-	
+
 	std::list<MeshBufList> &lists = drawbufs.lists;
-	
+
 	int timecheck_counter = 0;
 	for(std::list<MeshBufList>::iterator i = lists.begin();
 			i != lists.end(); ++i)
@@ -596,9 +608,9 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 		}
 
 		MeshBufList &list = *i;
-		
+
 		driver->setMaterial(list.m);
-		
+
 		for(std::list<scene::IMeshBuffer*>::iterator j = list.bufs.begin();
 				j != list.bufs.end(); ++j)
 		{
@@ -655,13 +667,13 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 #endif
 	}
 	} // ScopeProfiler
-	
+
 	// Log only on solid pass because values are the same
 	if(pass == scene::ESNRP_SOLID){
 		g_profiler->avg("CM: animated meshes", mesh_animate_count);
 		g_profiler->avg("CM: animated meshes (far)", mesh_animate_count_far);
 	}
-	
+
 	g_profiler->avg(prefix+"vertices drawn", vertex_count);
 	if(blocks_had_pass_meshbuf != 0)
 		g_profiler->avg(prefix+"meshbuffers per block",
@@ -712,7 +724,7 @@ static bool getVisibleBrightness(Map *map, v3f p0, v3f dir, float step,
 		pf += dir * step;
 		distance += step;
 		step *= step_multiplier;
-		
+
 		v3s16 p = floatToInt(pf, BS);
 		MapNode n = map->getNodeNoEx(p);
 		if(allow_allowing_non_sunlight_propagates && i == 0 &&
@@ -851,7 +863,7 @@ int ClientMap::getBackgroundBrightness(float max_d, u32 daylight_factor,
 	return ret;
 }
 
-void ClientMap::renderPostFx()
+void ClientMap::renderPostFx(CameraMode cam_mode)
 {
 	INodeDefManager *nodemgr = m_gamedef->ndef();
 
@@ -866,9 +878,12 @@ void ClientMap::renderPostFx()
 
 	// - If the player is in a solid node, make everything black.
 	// - If the player is in liquid, draw a semi-transparent overlay.
+	// - Do not if player is in third person mode
 	const ContentFeatures& features = nodemgr->get(n);
 	video::SColor post_effect_color = features.post_effect_color;
-	if(features.solidness == 2 && !(g_settings->getBool("noclip") && m_gamedef->checkLocalPrivilege("noclip")))
+	if(features.solidness == 2 && !(g_settings->getBool("noclip") &&
+			m_gamedef->checkLocalPrivilege("noclip")) &&
+			cam_mode == CAMERA_MODE_FIRST)
 	{
 		post_effect_color = video::SColor(255, 0, 0, 0);
 	}
