@@ -22,6 +22,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "irrlichttypes_extrabloated.h"
 #include "map.h"
+#include "camera.h"
 #include <set>
 #include <map>
 
@@ -86,12 +87,13 @@ public:
 		ISceneNode::drop();
 	}
 
-	void updateCamera(v3f pos, v3f dir, f32 fov)
+	void updateCamera(v3f pos, v3f dir, f32 fov, v3s16 offset)
 	{
 		JMutexAutoLock lock(m_camera_mutex);
 		m_camera_position = pos;
 		m_camera_direction = dir;
 		m_camera_fov = fov;
+		m_camera_offset = offset;
 	}
 
 	/*
@@ -125,7 +127,7 @@ public:
 	int getBackgroundBrightness(float max_d, u32 daylight_factor,
 			int oldvalue, bool *sunlight_seen_result);
 
-	void renderPostFx();
+	void renderPostFx(CameraMode cam_mode);
 
 	// For debug printing
 	virtual void PrintInfo(std::ostream &out);
@@ -146,6 +148,7 @@ private:
 	v3f m_camera_position;
 	v3f m_camera_direction;
 	f32 m_camera_fov;
+	v3s16 m_camera_offset;
 	JMutex m_camera_mutex;
 
 	std::map<v3s16, MapBlock*> m_drawlist;
