@@ -27,6 +27,8 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "inventory.h"
 #include "inventorymanager.h"
 #include "modalMenu.h"
+#include "guiTable.h"
+#include "clientserver.h"
 
 class IGameDef;
 class InventoryManager;
@@ -34,19 +36,29 @@ class ISimpleTextureSource;
 
 typedef enum {
 	f_Button,
-	f_ListBox,
+	f_Table,
 	f_TabHeader,
 	f_CheckBox,
 	f_DropDown,
 	f_Unknown
 } FormspecFieldType;
 
+typedef enum {
+	quit_mode_no,
+	quit_mode_accept,
+	quit_mode_cancel
+} FormspecQuitMode;
+
 struct TextDest
 {
 	virtual ~TextDest() {};
 	// This is deprecated I guess? -celeron55
 	virtual void gotText(std::wstring text){}
 	virtual void gotText(std::map<std::string, std::string> fields) = 0;
+	virtual void setFormName(std::string formname)
+	{ m_formname = formname;};
+
+	std::string m_formname;
 };
 
 class IFormSource
@@ -132,13 +144,14 @@ class GUIFormSpecMenu : public GUIModalMenu
 		v2s32 geom;
 		bool scale;
 	};
-	
+
 	struct FieldSpec
 	{
 		FieldSpec()
 		{
 		}
-		FieldSpec(const std::wstring name, const std::wstring label, const std::wstring fdeflt, int id):
+		FieldSpec(const std::wstring &name, const std::wstring &label,
+				const std::wstring &fdeflt, int id) :
 			fname(name),
 			flabel(label),
 			fdefault(fdeflt),
@@ -147,7 +160,6 @@ class GUIFormSpecMenu : public GUIModalMenu
 			send = false;
 			ftype = f_Unknown;
 			is_exit = false;
-			tooltip="";
 		}
 		std::wstring fname;
 		std::wstring flabel;
@@ -157,7 +169,6 @@ class GUIFormSpecMenu : public GUIModalMenu
 		FormspecFieldType ftype;
 		bool is_exit;
 		core::rect<s32> rect;
-		std::string tooltip;
 	};
 
 	struct BoxDrawSpec {
@@ -172,13 +183,32 @@ class GUIFormSpecMenu : public GUIModalMenu
 		irr::video::SColor color;
 	};
 
+	struct TooltipSpec {
+		TooltipSpec()
+		{
+		}
+		TooltipSpec(std::string a_tooltip, irr::video::SColor a_bgcolor,
+				irr::video::SColor a_color):
+			tooltip(a_tooltip),
+			bgcolor(a_bgcolor),
+			color(a_color)
+		{
+		}
+		std::string tooltip;
+		irr::video::SColor bgcolor;
+		irr::video::SColor color;
+	};
+
 public:
 	GUIFormSpecMenu(irr::IrrlichtDevice* dev,
 			gui::IGUIElement* parent, s32 id,
 			IMenuManager *menumgr,
 			InventoryManager *invmgr,
 			IGameDef *gamedef,
-			ISimpleTextureSource *tsrc
+			ISimpleTextureSource *tsrc,
+			IFormSource* fs_src,
+			TextDest* txt_dst,
+			GUIFormSpecMenu** ext_ptr
 			);
 
 	~GUIFormSpecMenu();
@@ -190,16 +220,22 @@ public:
 		m_current_inventory_location = current_inventory_location;
 		regenerateGui(m_screensize_old);
 	}
-	
+
 	// form_src is deleted by this GUIFormSpecMenu
 	void setFormSource(IFormSource *form_src)
 	{
+		if (m_form_src != NULL) {
+			delete m_form_src;
+		}
 		m_form_src = form_src;
 	}
 
 	// text_dst is deleted by this GUIFormSpecMenu
 	void setTextDest(TextDest *text_dst)
 	{
+		if (m_text_dst != NULL) {
+			delete m_text_dst;
+		}
 		m_text_dst = text_dst;
 	}
 
@@ -219,7 +255,7 @@ public:
 		Remove and re-add (or reposition) stuff
 	*/
 	void regenerateGui(v2u32 screensize);
-	
+
 	ItemSpec getItemAtPos(v2s32 p) const;
 	void drawList(const ListDrawSpec &s, int phase);
 	void drawSelectedItem();
@@ -227,11 +263,20 @@ public:
 	void updateSelectedItem();
 	ItemStack verifySelectedItem();
 
-	void acceptInput(bool quit);
+	void acceptInput(FormspecQuitMode quitmode);
 	bool preprocessEvent(const SEvent& event);
 	bool OnEvent(const SEvent& event);
+	bool doPause;
+	bool pausesGame() { return doPause; }
 
-	int getListboxIndex(std::string listboxname);
+	GUITable* getTable(std::wstring tablename);
+
+	static bool parseColor(const std::string &value,
+			video::SColor &color, bool quiet);
+
+#ifdef __ANDROID__
+	bool getAndroidUIInput();
+#endif
 
 protected:
 	v2s32 getBasePos() const
@@ -243,7 +288,7 @@ protected:
 	v2s32 spacing;
 	v2s32 imgsize;
 	v2s32 offset;
-	
+
 	irr::IrrlichtDevice* m_device;
 	InventoryManager *m_invmgr;
 	IGameDef *m_gamedef;
@@ -251,8 +296,6 @@ protected:
 
 	std::string m_formspec_string;
 	InventoryLocation m_current_inventory_location;
-	IFormSource *m_form_src;
-	TextDest *m_text_dst;
 
 	std::vector<ListDrawSpec> m_inventorylists;
 	std::vector<ImageDrawSpec> m_backgrounds;
@@ -260,28 +303,28 @@ protected:
 	std::vector<ImageDrawSpec> m_itemimages;
 	std::vector<BoxDrawSpec> m_boxes;
 	std::vector<FieldSpec> m_fields;
-	std::vector<std::pair<FieldSpec,gui::IGUIListBox*> > m_listboxes;
+	std::vector<std::pair<FieldSpec,GUITable*> > m_tables;
 	std::vector<std::pair<FieldSpec,gui::IGUICheckBox*> > m_checkboxes;
-
+	std::map<std::wstring, TooltipSpec> m_tooltips;
+	
 	ItemSpec *m_selected_item;
 	u32 m_selected_amount;
 	bool m_selected_dragging;
-	
+
 	// WARNING: BLACK MAGIC
 	// Used to guess and keep up with some special things the server can do.
 	// If name is "", no guess exists.
 	ItemStack m_selected_content_guess;
 	InventoryLocation m_selected_content_guess_inventory;
 
-	// WARNING: BLACK IRRLICHT MAGIC, see checkListboxClick()
-	std::wstring m_listbox_click_fname;
-	int m_listbox_click_index;
-	u32 m_listbox_click_time;
-	bool m_listbox_doubleclick;
-
 	v2s32 m_pointer;
 	gui::IGUIStaticText *m_tooltip_element;
 
+	u32 m_tooltip_show_delay;
+	s32 m_hoovered_time;
+	s32 m_old_tooltip_id;
+	std::string m_old_tooltip;
+
 	bool m_allowclose;
 	bool m_lock;
 	v2u32 m_lockscreensize;
@@ -293,17 +336,27 @@ protected:
 	video::SColor m_slotbg_n;
 	video::SColor m_slotbg_h;
 	video::SColor m_slotbordercolor;
+	video::SColor m_default_tooltip_bgcolor;
+	video::SColor m_default_tooltip_color;
+
 private:
+	IFormSource      *m_form_src;
+	TextDest         *m_text_dst;
+	GUIFormSpecMenu **m_ext_ptr;
+	gui::IGUIFont    *m_font;
+	unsigned int      m_formspec_version;
+
 	typedef struct {
 		v2s32 size;
-		s32 helptext_h;
 		core::rect<s32> rect;
 		v2s32 basepos;
 		int bp_set;
 		v2u32 screensize;
 		std::wstring focused_fieldname;
-		std::map<std::wstring,int> listbox_selections;
-		std::map<std::wstring,int> listbox_scroll;
+		GUITable::TableOptions table_options;
+		GUITable::TableColumns table_columns;
+		// used to restore table selection/scroll/treeview state
+		std::map<std::wstring,GUITable::DynamicData> table_dyndata;
 	} parserData;
 
 	typedef struct {
@@ -315,12 +368,6 @@ private:
 
 	fs_key_pendig current_keys_pending;
 
-	// Determine whether listbox click was double click
-	// (Using some black Irrlicht magic)
-	bool checkListboxClick(std::wstring wlistboxname, int eventtype);
-
-	gui::IGUIScrollBar* getListboxScrollbar(gui::IGUIListBox *listbox);
-
 	void parseElement(parserData* data,std::string element);
 
 	void parseSize(parserData* data,std::string element);
@@ -330,12 +377,16 @@ private:
 	void parseItemImage(parserData* data,std::string element);
 	void parseButton(parserData* data,std::string element,std::string typ);
 	void parseBackground(parserData* data,std::string element);
+	void parseTableOptions(parserData* data,std::string element);
+	void parseTableColumns(parserData* data,std::string element);
+	void parseTable(parserData* data,std::string element);
 	void parseTextList(parserData* data,std::string element);
 	void parseDropDown(parserData* data,std::string element);
 	void parsePwdField(parserData* data,std::string element);
 	void parseField(parserData* data,std::string element,std::string type);
 	void parseSimpleField(parserData* data,std::vector<std::string> &parts);
-	void parseTextArea(parserData* data,std::vector<std::string>& parts,std::string type);
+	void parseTextArea(parserData* data,std::vector<std::string>& parts,
+			std::string type);
 	void parseLabel(parserData* data,std::string element);
 	void parseVertLabel(parserData* data,std::string element);
 	void parseImageButton(parserData* data,std::string element,std::string type);
@@ -344,26 +395,47 @@ private:
 	void parseBox(parserData* data,std::string element);
 	void parseBackgroundColor(parserData* data,std::string element);
 	void parseListColors(parserData* data,std::string element);
+	void parseTooltip(parserData* data,std::string element);
+	bool parseVersionDirect(std::string data);
+
+	/**
+	 * check if event is part of a double click
+	 * @param event event to evaluate
+	 * @return true/false if a doubleclick was detected
+	 */
+	bool DoubleClickDetection(const SEvent event);
+
+	struct clickpos
+	{
+		v2s32 pos;
+		s32 time;
+	};
+	clickpos m_doubleclickdetect[2];
+
+	int m_btn_height;
+
+	std::wstring getLabelByID(s32 id);
+	std::wstring getNameByID(s32 id);
+#ifdef __ANDROID__
+	v2s32 m_down_pos;
+	std::wstring m_JavaDialogFieldName;
+#endif
 
-	bool parseColor(std::string &value, video::SColor &color, bool quiet);
 };
 
 class FormspecFormSource: public IFormSource
 {
 public:
-	FormspecFormSource(std::string formspec,FormspecFormSource** game_formspec)
+	FormspecFormSource(std::string formspec)
 	{
 		m_formspec = formspec;
-		m_game_formspec = game_formspec;
 	}
 
 	~FormspecFormSource()
-	{
-		*m_game_formspec = 0;
-	}
+	{}
 
 	void setForm(std::string formspec) {
-		m_formspec = formspec;
+		m_formspec = FORMSPEC_VERSION_STRING + formspec;
 	}
 
 	std::string getForm()
@@ -372,7 +444,6 @@ public:
 	}
 
 	std::string m_formspec;
-	FormspecFormSource** m_game_formspec;
 };
 
 #endif
