@@ -22,7 +22,16 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 #if defined(WIN32)
 #include <windows.h>
+#include <assert.h>
 #define MAX_SEMAPHORE_COUNT 1024
+#elif __MACH__
+#include <pthread.h>
+#include <mach/mach.h>
+#include <mach/task.h>
+#include <mach/semaphore.h>
+#include <sys/semaphore.h>
+#include <errno.h>
+#include <time.h>
 #else
 #include <pthread.h>
 #include <semaphore.h>
@@ -36,15 +45,20 @@ public:
 
 	void Post();
 	void Wait();
+	bool Wait(unsigned int time_ms);
 
 	int GetValue();
 
 private:
 #if defined(WIN32)
 	HANDLE m_hSemaphore;
+#elif __MACH__
+	semaphore_t m_semaphore;
+	int semcount;
 #else
 	sem_t m_semaphore;
 #endif
 };
 
+
 #endif /* JSEMAPHORE_H_ */
