@@ -25,9 +25,17 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 */
 #include "debug.h" // assert
 #include "modalMenu.h"
-#include "guiPauseMenu.h" //For IGameCallback
 #include <list>
 
+class IGameCallback
+{
+public:
+	virtual void exitToOS() = 0;
+	virtual void disconnect() = 0;
+	virtual void changePassword() = 0;
+	virtual void changeVolume() = 0;
+};
+
 extern gui::IGUIEnvironment* guienv;
 extern gui::IGUIStaticText *guiroot;
 
@@ -91,6 +99,17 @@ public:
 		return m_stack.size();
 	}
 
+	bool pausesGame()
+	{
+		for(std::list<GUIModalMenu*>::iterator
+				i = m_stack.begin(); i != m_stack.end(); ++i)
+		{
+			if((*i)->pausesGame())
+				return true;
+		}
+		return false;
+	}
+
 	std::list<GUIModalMenu*> m_stack;
 };
 
@@ -105,13 +124,17 @@ public:
 		disconnect_requested(false),
 		changepassword_requested(false),
 		changevolume_requested(false),
+		shutdown_requested(false),
 		device(a_device)
 	{
 	}
 
 	virtual void exitToOS()
 	{
+		shutdown_requested = true;
+#ifndef __ANDROID__
 		device->closeDevice();
+#endif
 	}
 
 	virtual void disconnect()
@@ -132,6 +155,7 @@ public:
 	bool disconnect_requested;
 	bool changepassword_requested;
 	bool changevolume_requested;
+	bool shutdown_requested;
 	IrrlichtDevice *device;
 };
 
