@@ -513,11 +513,6 @@ public:
 	NodeMetadataList m_node_metadata;
 	NodeTimerList m_node_timers;
 	StaticObjectList m_static_objects;
-	
-	s16 heat;
-	s16 humidity;
-	u32 heat_last_update;
-	u32 humidity_last_update;
 
 private:
 	/*
