@@ -34,22 +34,24 @@ bool ScriptApiItem::item_OnDrop(ItemStack &item,
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	// Push callback function on stack
-	if(!getItemCallback(item.name.c_str(), "on_drop"))
+	if (!getItemCallback(item.name.c_str(), "on_drop"))
 		return false;
 
 	// Call function
 	LuaItemStack::create(L, item);
 	objectrefGetOrCreate(dropper);
 	pushFloatPos(L, pos);
-	if(lua_pcall(L, 3, 1, errorhandler))
+	if (lua_pcall(L, 3, 1, m_errorhandler))
 		scriptError();
-	if(!lua_isnil(L, -1))
-		item = read_item(L,-1, getServer());
-	lua_pop(L, 2);  // Pop item and error handler
+	if (!lua_isnil(L, -1)) {
+		try {
+			item = read_item(L,-1, getServer());
+		} catch (LuaError &e) {
+			throw LuaError(std::string(e.what()) + ". item=" + item.name);
+		}
+	}
+	lua_pop(L, 1);  // Pop item
 	return true;
 }
 
@@ -58,22 +60,24 @@ bool ScriptApiItem::item_OnPlace(ItemStack &item,
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	// Push callback function on stack
-	if(!getItemCallback(item.name.c_str(), "on_place"))
+	if (!getItemCallback(item.name.c_str(), "on_place"))
 		return false;
 
 	// Call function
 	LuaItemStack::create(L, item);
 	objectrefGetOrCreate(placer);
 	pushPointedThing(pointed);
-	if(lua_pcall(L, 3, 1, errorhandler))
+	if (lua_pcall(L, 3, 1, m_errorhandler))
 		scriptError();
-	if(!lua_isnil(L, -1))
-		item = read_item(L,-1, getServer());
-	lua_pop(L, 2);  // Pop item and error handler
+	if (!lua_isnil(L, -1)) {
+		try {
+			item = read_item(L,-1, getServer());
+		} catch (LuaError &e) {
+			throw LuaError(std::string(e.what()) + ". item=" + item.name);
+		}
+	}
+	lua_pop(L, 1);  // Pop item
 	return true;
 }
 
@@ -82,22 +86,24 @@ bool ScriptApiItem::item_OnUse(ItemStack &item,
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
 	// Push callback function on stack
-	if(!getItemCallback(item.name.c_str(), "on_use"))
+	if (!getItemCallback(item.name.c_str(), "on_use"))
 		return false;
 
 	// Call function
 	LuaItemStack::create(L, item);
 	objectrefGetOrCreate(user);
 	pushPointedThing(pointed);
-	if(lua_pcall(L, 3, 1, errorhandler))
+	if (lua_pcall(L, 3, 1, m_errorhandler))
 		scriptError();
-	if(!lua_isnil(L, -1))
-		item = read_item(L,-1, getServer());
-	lua_pop(L, 2);  // Pop item and error handler
+	if(!lua_isnil(L, -1)) {
+		try {
+			item = read_item(L,-1, getServer());
+		} catch (LuaError &e) {
+			throw LuaError(std::string(e.what()) + ". item=" + item.name);
+		}
+	}
+	lua_pop(L, 1);  // Pop item
 	return true;
 }
 
@@ -106,26 +112,29 @@ bool ScriptApiItem::item_OnCraft(ItemStack &item, ServerActiveObject *user,
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
-	lua_getglobal(L, "minetest");
+	lua_getglobal(L, "core");
 	lua_getfield(L, -1, "on_craft");
 	LuaItemStack::create(L, item);
 	objectrefGetOrCreate(user);
 	
-	//Push inventory list
+	// Push inventory list
 	std::vector<ItemStack> items;
-	for(u32 i=0; i<old_craft_grid->getSize(); i++)
+	for (u32 i = 0; i < old_craft_grid->getSize(); i++) {
 		items.push_back(old_craft_grid->getItem(i));
+	}
 	push_items(L, items);
 
 	InvRef::create(L, craft_inv);
-	if(lua_pcall(L, 4, 1, errorhandler))
+	if (lua_pcall(L, 4, 1, m_errorhandler))
 		scriptError();
-	if(!lua_isnil(L, -1))
-		item = read_item(L,-1, getServer());
-	lua_pop(L, 2);  // Pop item and error handler
+	if (!lua_isnil(L, -1)) {
+		try {
+			item = read_item(L,-1, getServer());
+		} catch (LuaError &e) {
+			throw LuaError(std::string(e.what()) + ". item=" + item.name);
+		}
+	}
+	lua_pop(L, 1);  // Pop item
 	return true;
 }
 
@@ -134,42 +143,45 @@ bool ScriptApiItem::item_CraftPredict(ItemStack &item, ServerActiveObject *user,
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	lua_pushcfunction(L, script_error_handler);
-	int errorhandler = lua_gettop(L);
-
-	lua_getglobal(L, "minetest");
+	lua_getglobal(L, "core");
 	lua_getfield(L, -1, "craft_predict");
 	LuaItemStack::create(L, item);
 	objectrefGetOrCreate(user);
 
 	//Push inventory list
 	std::vector<ItemStack> items;
-	for(u32 i=0; i<old_craft_grid->getSize(); i++)
+	for (u32 i = 0; i < old_craft_grid->getSize(); i++) {
 		items.push_back(old_craft_grid->getItem(i));
+	}
 	push_items(L, items);
 
 	InvRef::create(L, craft_inv);
-	if(lua_pcall(L, 4, 1, errorhandler))
+	if (lua_pcall(L, 4, 1, m_errorhandler))
 		scriptError();
-	if(!lua_isnil(L, -1))
-		item = read_item(L,-1, getServer());
-	lua_pop(L, 2);  // Pop item and error handler
+	if (!lua_isnil(L, -1)) {
+		try {
+			item = read_item(L,-1, getServer());
+		} catch (LuaError &e) {
+			throw LuaError(std::string(e.what()) + ". item=" + item.name);
+		}
+	}
+	lua_pop(L, 1);  // Pop item
 	return true;
 }
 
-// Retrieves minetest.registered_items[name][callbackname]
+// Retrieves core.registered_items[name][callbackname]
 // If that is nil or on error, return false and stack is unchanged
 // If that is a function, returns true and pushes the
 // function onto the stack
-// If minetest.registered_items[name] doesn't exist, minetest.nodedef_default
+// If core.registered_items[name] doesn't exist, core.nodedef_default
 // is tried instead so unknown items can still be manipulated to some degree
 bool ScriptApiItem::getItemCallback(const char *name, const char *callbackname)
 {
 	lua_State* L = getStack();
 
-	lua_getglobal(L, "minetest");
+	lua_getglobal(L, "core");
 	lua_getfield(L, -1, "registered_items");
-	lua_remove(L, -2); // Remove minetest
+	lua_remove(L, -2); // Remove core
 	luaL_checktype(L, -1, LUA_TTABLE);
 	lua_getfield(L, -1, name);
 	lua_remove(L, -2); // Remove registered_items
@@ -180,8 +192,8 @@ bool ScriptApiItem::getItemCallback(const char *name, const char *callbackname)
 		errorstream << "Item \"" << name << "\" not defined" << std::endl;
 		lua_pop(L, 1);
 
-		// Try minetest.nodedef_default instead
-		lua_getglobal(L, "minetest");
+		// Try core.nodedef_default instead
+		lua_getglobal(L, "core");
 		lua_getfield(L, -1, "nodedef_default");
 		lua_remove(L, -2);
 		luaL_checktype(L, -1, LUA_TTABLE);
@@ -227,4 +239,3 @@ void ScriptApiItem::pushPointedThing(const PointedThing& pointed)
 	}
 }
 
-
