@@ -25,131 +25,136 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 class ModApiEnvMod : public ModApiBase {
 private:
-	// minetest.set_node(pos, node)
+	// set_node(pos, node)
 	// pos = {x=num, y=num, z=num}
 	static int l_set_node(lua_State *L);
 
 	static int l_add_node(lua_State *L);
 
-	// minetest.remove_node(pos)
+	// remove_node(pos)
 	// pos = {x=num, y=num, z=num}
 	static int l_remove_node(lua_State *L);
 	
-	// minetest.swap_node(pos, node)
+	// swap_node(pos, node)
 	// pos = {x=num, y=num, z=num}
 	static int l_swap_node(lua_State *L);
 
-	// minetest.get_node(pos)
+	// get_node(pos)
 	// pos = {x=num, y=num, z=num}
 	static int l_get_node(lua_State *L);
 
-	// minetest.get_node_or_nil(pos)
+	// get_node_or_nil(pos)
 	// pos = {x=num, y=num, z=num}
 	static int l_get_node_or_nil(lua_State *L);
 
-	// minetest.get_node_light(pos, timeofday)
+	// get_node_light(pos, timeofday)
 	// pos = {x=num, y=num, z=num}
 	// timeofday: nil = current time, 0 = night, 0.5 = day
 	static int l_get_node_light(lua_State *L);
 
-	// minetest.place_node(pos, node)
+	// place_node(pos, node)
 	// pos = {x=num, y=num, z=num}
 	static int l_place_node(lua_State *L);
 
-	// minetest.dig_node(pos)
+	// dig_node(pos)
 	// pos = {x=num, y=num, z=num}
 	static int l_dig_node(lua_State *L);
 
-	// minetest.punch_node(pos)
+	// punch_node(pos)
 	// pos = {x=num, y=num, z=num}
 	static int l_punch_node(lua_State *L);
 
 
-	// minetest.get_node_max_level(pos)
+	// get_node_max_level(pos)
 	// pos = {x=num, y=num, z=num}
 	static int l_get_node_max_level(lua_State *L);
 
-	// minetest.get_node_level(pos)
+	// get_node_level(pos)
 	// pos = {x=num, y=num, z=num}
 	static int l_get_node_level(lua_State *L);
 
-	// minetest.set_node_level(pos)
+	// set_node_level(pos)
 	// pos = {x=num, y=num, z=num}
 	static int l_set_node_level(lua_State *L);
 
-	// minetest.add_node_level(pos)
+	// add_node_level(pos)
 	// pos = {x=num, y=num, z=num}
 	static int l_add_node_level(lua_State *L);
 
-	// minetest.get_meta(pos)
+	// get_meta(pos)
 	static int l_get_meta(lua_State *L);
 
-	// minetest.get_node_timer(pos)
+	// get_node_timer(pos)
 	static int l_get_node_timer(lua_State *L);
 
-	// minetest.add_entity(pos, entityname) -> ObjectRef or nil
+	// add_entity(pos, entityname) -> ObjectRef or nil
 	// pos = {x=num, y=num, z=num}
 	static int l_add_entity(lua_State *L);
 
-	// minetest.add_item(pos, itemstack or itemstring or table) -> ObjectRef or nil
+	// add_item(pos, itemstack or itemstring or table) -> ObjectRef or nil
 	// pos = {x=num, y=num, z=num}
 	static int l_add_item(lua_State *L);
 
-	// minetest.get_player_by_name(name)
+	// get_player_by_name(name)
 	static int l_get_player_by_name(lua_State *L);
 
-	// minetest.get_objects_inside_radius(pos, radius)
+	// get_objects_inside_radius(pos, radius)
 	static int l_get_objects_inside_radius(lua_State *L);
 
-	// minetest.set_timeofday(val)
+	// set_timeofday(val)
 	// val = 0...1
 	static int l_set_timeofday(lua_State *L);
 
-	// minetest.get_timeofday() -> 0...1
+	// get_timeofday() -> 0...1
 	static int l_get_timeofday(lua_State *L);
 
-	// minetest.get_gametime()
+	// get_gametime()
 	static int l_get_gametime(lua_State *L);
 
-	// minetest.find_node_near(pos, radius, nodenames) -> pos or nil
+	// find_node_near(pos, radius, nodenames) -> pos or nil
 	// nodenames: eg. {"ignore", "group:tree"} or "default:dirt"
 	static int l_find_node_near(lua_State *L);
 
-	// minetest.find_nodes_in_area(minp, maxp, nodenames) -> list of positions
+	// find_nodes_in_area(minp, maxp, nodenames) -> list of positions
 	// nodenames: eg. {"ignore", "group:tree"} or "default:dirt"
 	static int l_find_nodes_in_area(lua_State *L);
 
-	// minetest.get_perlin(seeddiff, octaves, persistence, scale)
+	// get_perlin(seeddiff, octaves, persistence, scale)
 	// returns world-specific PerlinNoise
 	static int l_get_perlin(lua_State *L);
 
-	// minetest.get_perlin_map(noiseparams, size)
+	// get_perlin_map(noiseparams, size)
 	// returns world-specific PerlinNoiseMap
 	static int l_get_perlin_map(lua_State *L);
 	
-	// minetest.get_voxel_manip()
+	// get_voxel_manip()
 	// returns world-specific voxel manipulator
 	static int l_get_voxel_manip(lua_State *L);
 	
-	// minetest.clear_objects()
+	// clear_objects()
 	// clear all objects in the environment
 	static int l_clear_objects(lua_State *L);
 
-	// minetest.spawn_tree(pos, treedef)
+	// spawn_tree(pos, treedef)
 	static int l_spawn_tree(lua_State *L);
 
-	// minetest.line_of_sight(pos1, pos2, stepsize) -> true/false
+	// line_of_sight(pos1, pos2, stepsize) -> true/false
 	static int l_line_of_sight(lua_State *L);
 
-	// minetest.find_path(pos1, pos2, searchdistance,
+	// find_path(pos1, pos2, searchdistance,
 	//     max_jump, max_drop, algorithm) -> table containing path
 	static int l_find_path(lua_State *L);
 
-	// minetest.transforming_liquid_add(pos)
+	// transforming_liquid_add(pos)
 	static int l_transforming_liquid_add(lua_State *L);
 
-	static int l_get_heat(lua_State *L);
-	static int l_get_humidity(lua_State *L);
+	// forceload_block(blockpos)
+	// forceloads a block
+	static int l_forceload_block(lua_State *L);
+	
+	// forceload_free_block(blockpos)
+	// stops forceloading a position
+	static int l_forceload_free_block(lua_State *L);
 	
 public:
 	static void Initialize(lua_State *L, int top);
