@@ -38,6 +38,7 @@ struct EnumString es_HudElementType[] =
 	{HUD_ELEM_TEXT,      "text"},
 	{HUD_ELEM_STATBAR,   "statbar"},
 	{HUD_ELEM_INVENTORY, "inventory"},
+	{HUD_ELEM_WAYPOINT,  "waypoint"},
 {0, NULL},
 };
 
@@ -53,6 +54,7 @@ struct EnumString es_HudElementStat[] =
 	{HUD_STAT_DIR,    "direction"},
 	{HUD_STAT_ALIGN,  "alignment"},
 	{HUD_STAT_OFFSET, "offset"},
+	{HUD_STAT_WORLD_POS, "world_pos"},
 	{0, NULL},
 };
 
@@ -404,6 +406,61 @@ int ObjectRef::l_set_animation(lua_State *L)
 	return 0;
 }
 
+// set_local_animation(self, {stand/idle}, {walk}, {dig}, {walk+dig}, frame_speed)
+int ObjectRef::l_set_local_animation(lua_State *L)
+{
+	//NO_MAP_LOCK_REQUIRED;
+	ObjectRef *ref = checkobject(L, 1);
+	Player *player = getplayer(ref);
+	if (player == NULL)
+		return 0;
+	// Do it
+	v2s32 frames[4];
+	for (int i=0;i<4;i++) {
+		if(!lua_isnil(L, 2+1))
+			frames[i] = read_v2s32(L, 2+i);
+	}
+	float frame_speed = 30;
+	if(!lua_isnil(L, 6))
+		frame_speed = lua_tonumber(L, 6);
+
+	if (!getServer(L)->setLocalPlayerAnimations(player, frames, frame_speed))
+		return 0;
+
+	lua_pushboolean(L, true);
+	return 0;
+}
+
+// set_eye_offset(self, v3f first pv, v3f third pv)
+int ObjectRef::l_set_eye_offset(lua_State *L)
+{
+	//NO_MAP_LOCK_REQUIRED;
+	ObjectRef *ref = checkobject(L, 1);
+	Player *player = getplayer(ref);
+	if (player == NULL)
+		return 0;
+	// Do it
+	v3f offset_first = v3f(0, 0, 0);
+	v3f offset_third = v3f(0, 0, 0);
+	
+	if(!lua_isnil(L, 2))
+		offset_first = read_v3f(L, 2);
+	if(!lua_isnil(L, 3))
+		offset_third = read_v3f(L, 3);
+
+	// Prevent abuse of offset values (keep player always visible)
+	offset_third.X = rangelim(offset_third.X,-10,10);
+	offset_third.Z = rangelim(offset_third.Z,-5,5);
+	/* TODO: if possible: improve the camera colision detetion to allow Y <= -1.5) */
+	offset_third.Y = rangelim(offset_third.Y,-10,15); //1.5*BS
+
+	if (!getServer(L)->setPlayerEyeOffset(player, offset_first, offset_third))
+		return 0;
+
+	lua_pushboolean(L, true);
+	return 0;
+}
+
 // set_bone_position(self, std::string bone, v3f position, v3f rotation)
 int ObjectRef::l_set_bone_position(lua_State *L)
 {
@@ -602,6 +659,7 @@ int ObjectRef::l_get_entity_name(lua_State *L)
 	NO_MAP_LOCK_REQUIRED;
 	ObjectRef *ref = checkobject(L, 1);
 	LuaEntitySAO *co = getluaobject(ref);
+	log_deprecated(L,"Deprecated call to \"get_entity_name");
 	if(co == NULL) return 0;
 	// Do it
 	std::string name = co->getName();
@@ -848,6 +906,10 @@ int ObjectRef::l_hud_add(lua_State *L)
 	elem->scale = lua_istable(L, -1) ? read_v2f(L, -1) : v2f();
 	lua_pop(L, 1);
 
+	lua_getfield(L, 2, "size");
+	elem->size = lua_istable(L, -1) ? read_v2s32(L, -1) : v2s32();
+	lua_pop(L, 1);
+
 	elem->name   = getstringfield_default(L, 2, "name", "");
 	elem->text   = getstringfield_default(L, 2, "text", "");
 	elem->number = getintfield_default(L, 2, "number", 0);
@@ -862,6 +924,15 @@ int ObjectRef::l_hud_add(lua_State *L)
 	elem->offset = lua_istable(L, -1) ? read_v2f(L, -1) : v2f();
 	lua_pop(L, 1);
 
+	lua_getfield(L, 2, "world_pos");
+	elem->world_pos = lua_istable(L, -1) ? read_v3f(L, -1) : v3f();
+	lua_pop(L, 1);
+
+	/* check for known deprecated element usage */
+	if ((elem->type  == HUD_ELEM_STATBAR) && (elem->size == v2s32())) {
+		log_deprecated(L,"Deprecated usage of statbar without size!");
+	}
+
 	u32 id = getServer(L)->hudAdd(player, elem);
 	if (id == (u32)-1) {
 		delete elem;
@@ -900,7 +971,9 @@ int ObjectRef::l_hud_change(lua_State *L)
 		return 0;
 
 	u32 id = !lua_isnil(L, 2) ? lua_tonumber(L, 2) : -1;
-	if (id >= player->hud.size())
+
+	HudElement *e = player->getHud(id);
+	if (!e)
 		return 0;
 
 	HudElementStat stat = HUD_STAT_NUMBER;
@@ -912,10 +985,6 @@ int ObjectRef::l_hud_change(lua_State *L)
 	}
 
 	void *value = NULL;
-	HudElement *e = player->hud[id];
-	if (!e)
-		return 0;
-
 	switch (stat) {
 		case HUD_STAT_POS:
 			e->pos = read_v2f(L, 4);
@@ -944,12 +1013,23 @@ int ObjectRef::l_hud_change(lua_State *L)
 		case HUD_STAT_DIR:
 			e->dir = lua_tonumber(L, 4);
 			value = &e->dir;
+			break;
 		case HUD_STAT_ALIGN:
 			e->align = read_v2f(L, 4);
 			value = &e->align;
+			break;
 		case HUD_STAT_OFFSET:
 			e->offset = read_v2f(L, 4);
 			value = &e->offset;
+			break;
+		case HUD_STAT_WORLD_POS:
+			e->world_pos = read_v3f(L, 4);
+			value = &e->world_pos;
+			break;
+		case HUD_STAT_SIZE:
+			e->size = read_v2s32(L, 4);
+			value = &e->size;
+			break;
 	}
 
 	getServer(L)->hudChange(player, id, stat, value);
@@ -967,10 +1047,8 @@ int ObjectRef::l_hud_get(lua_State *L)
 		return 0;
 
 	u32 id = lua_tonumber(L, -1);
-	if (id >= player->hud.size())
-		return 0;
 
-	HudElement *e = player->hud[id];
+	HudElement *e = player->getHud(id);
 	if (!e)
 		return 0;
 
@@ -1000,6 +1078,9 @@ int ObjectRef::l_hud_get(lua_State *L)
 	lua_pushnumber(L, e->dir);
 	lua_setfield(L, -2, "dir");
 
+	push_v3f(L, e->world_pos);
+	lua_setfield(L, -2, "world_pos");
+
 	return 1;
 }
 
@@ -1029,6 +1110,28 @@ int ObjectRef::l_hud_set_flags(lua_State *L)
 	return 1;
 }
 
+int ObjectRef::l_hud_get_flags(lua_State *L)
+{
+	ObjectRef *ref = checkobject(L, 1);
+	Player *player = getplayer(ref);
+	if (player == NULL)
+		return 0;
+
+	lua_newtable(L);
+	lua_pushboolean(L, player->hud_flags & HUD_FLAG_HOTBAR_VISIBLE);
+	lua_setfield(L, -2, "hotbar");
+	lua_pushboolean(L, player->hud_flags & HUD_FLAG_HEALTHBAR_VISIBLE);
+	lua_setfield(L, -2, "healthbar");
+	lua_pushboolean(L, player->hud_flags & HUD_FLAG_CROSSHAIR_VISIBLE);
+	lua_setfield(L, -2, "crosshair");
+	lua_pushboolean(L, player->hud_flags & HUD_FLAG_WIELDITEM_VISIBLE);
+	lua_setfield(L, -2, "wielditem");
+	lua_pushboolean(L, player->hud_flags & HUD_FLAG_BREATHBAR_VISIBLE);
+	lua_setfield(L, -2, "breathbar");
+
+	return 1;
+}
+
 // hud_set_hotbar_itemcount(self, hotbar_itemcount)
 int ObjectRef::l_hud_set_hotbar_itemcount(lua_State *L)
 {
@@ -1074,6 +1177,67 @@ int ObjectRef::l_hud_set_hotbar_selected_image(lua_State *L)
 	return 1;
 }
 
+// set_sky(self, bgcolor, type, list)
+int ObjectRef::l_set_sky(lua_State *L)
+{
+	ObjectRef *ref = checkobject(L, 1);
+	Player *player = getplayer(ref);
+	if (player == NULL)
+		return 0;
+
+	video::SColor bgcolor(255,255,255,255);
+	if (!lua_isnil(L, 2))
+		bgcolor = readARGB8(L, 2);
+
+	std::string type = luaL_checkstring(L, 3);
+
+	std::vector<std::string> params;
+	if (lua_istable(L, 4)) {
+		int table = lua_gettop(L);
+		lua_pushnil(L);
+		while (lua_next(L, table) != 0) {
+			// key at index -2 and value at index -1
+			if (lua_isstring(L, -1))
+				params.push_back(lua_tostring(L, -1));
+			else
+				params.push_back("");
+			// removes value, keeps key for next iteration
+			lua_pop(L, 1);
+		}
+	}
+
+	if (type == "skybox" && params.size() != 6)
+		throw LuaError("skybox expects 6 textures");
+
+	if (!getServer(L)->setSky(player, bgcolor, type, params))
+		return 0;
+
+	lua_pushboolean(L, true);
+	return 1;
+}
+
+// override_day_night_ratio(self, brightness=0...1)
+int ObjectRef::l_override_day_night_ratio(lua_State *L)
+{
+	ObjectRef *ref = checkobject(L, 1);
+	Player *player = getplayer(ref);
+	if (player == NULL)
+		return 0;
+
+	bool do_override = false;
+	float ratio = 0.0f;
+	if (!lua_isnil(L, 2)){
+		do_override = true;
+		ratio = luaL_checknumber(L, 2);
+	}
+
+	if (!getServer(L)->overrideDayNightRatio(player, do_override, ratio))
+		return 0;
+
+	lua_pushboolean(L, true);
+	return 1;
+}
+
 ObjectRef::ObjectRef(ServerActiveObject *object):
 	m_object(object)
 {
@@ -1188,8 +1352,13 @@ const luaL_reg ObjectRef::methods[] = {
 	luamethod(ObjectRef, hud_change),
 	luamethod(ObjectRef, hud_get),
 	luamethod(ObjectRef, hud_set_flags),
+	luamethod(ObjectRef, hud_get_flags),
 	luamethod(ObjectRef, hud_set_hotbar_itemcount),
 	luamethod(ObjectRef, hud_set_hotbar_image),
 	luamethod(ObjectRef, hud_set_hotbar_selected_image),
+	luamethod(ObjectRef, set_sky),
+	luamethod(ObjectRef, override_day_night_ratio),
+	luamethod(ObjectRef, set_local_animation),
+	luamethod(ObjectRef, set_eye_offset),
 	{0,0}
 };
