@@ -29,6 +29,58 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "../hex.h"
 #include "../porting.h"
 
+#ifdef __ANDROID__
+const wchar_t* wide_chars = L" !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";
+
+int wctomb(char *s, wchar_t wc)
+{
+	for (unsigned int j = 0; j < (sizeof(wide_chars)/sizeof(wchar_t));j++) {
+		if (wc == wide_chars[j]) {
+			*s = (char) (j+32);
+			return 1;
+		}
+		else if (wc == L'\n') {
+			*s = '\n';
+			return 1;
+		}
+	}
+	return -1;
+}
+
+int mbtowc(wchar_t *pwc, const char *s, size_t n)
+{
+	std::wstring intermediate = narrow_to_wide(s);
+
+	if (intermediate.length() > 0) {
+		*pwc = intermediate[0];
+		return 1;
+	}
+	else {
+		return -1;
+	}
+}
+
+std::wstring narrow_to_wide(const std::string& mbs) {
+	size_t wcl = mbs.size();
+
+	std::wstring retval = L"";
+
+	for (unsigned int i = 0; i < wcl; i++) {
+		if (((unsigned char) mbs[i] >31) &&
+		 ((unsigned char) mbs[i] < 127)) {
+
+			retval += wide_chars[(unsigned char) mbs[i] -32];
+		}
+		//handle newline
+		else if (mbs[i] == '\n') {
+			retval += L'\n';
+		}
+	}
+
+	return retval;
+}
+#else
+
 std::wstring narrow_to_wide(const std::string& mbs)
 {
 	size_t wcl = mbs.size();
@@ -40,6 +92,35 @@ std::wstring narrow_to_wide(const std::string& mbs)
 	return *wcs;
 }
 
+#endif
+
+#ifdef __ANDROID__
+std::string wide_to_narrow(const std::wstring& wcs) {
+	size_t mbl = wcs.size()*4;
+
+	std::string retval = "";
+	for (unsigned int i = 0; i < wcs.size(); i++) {
+		wchar_t char1 = (wchar_t) wcs[i];
+
+		if (char1 == L'\n') {
+			retval += '\n';
+			continue;
+		}
+
+		for (unsigned int j = 0; j < wcslen(wide_chars);j++) {
+			wchar_t char2 = (wchar_t) wide_chars[j];
+
+			if (char1 == char2) {
+				char toadd = (j+32);
+				retval += toadd;
+				break;
+			}
+		}
+	}
+
+	return retval;
+}
+#else
 std::string wide_to_narrow(const std::wstring& wcs)
 {
 	size_t mbl = wcs.size()*4;
@@ -53,6 +134,8 @@ std::string wide_to_narrow(const std::wstring& wcs)
 	return *mbs;
 }
 
+#endif
+
 // Get an sha-1 hash of the player's name combined with
 // the password entered. That's what the server uses as
 // their password. (Exception : if the password field is
@@ -108,46 +191,76 @@ std::string urldecode(std::string str)
 	return oss.str();
 }
 
-u32 readFlagString(std::string str, FlagDesc *flagdesc) {
-	u32 result = 0;
+u32 readFlagString(std::string str, FlagDesc *flagdesc, u32 *flagmask)
+{
+	u32 result = 0, mask = 0;
 	char *s = &str[0];
 	char *flagstr, *strpos = NULL;
-	
+
 	while ((flagstr = strtok_r(s, ",", &strpos))) {
 		s = NULL;
-		
+
 		while (*flagstr == ' ' || *flagstr == '\t')
 			flagstr++;
-		
+
+		bool flagset = true;
+		if (!strncasecmp(flagstr, "no", 2)) {
+			flagset = false;
+			flagstr += 2;
+		}
+
 		for (int i = 0; flagdesc[i].name; i++) {
 			if (!strcasecmp(flagstr, flagdesc[i].name)) {
-				result |= flagdesc[i].flag;
+				mask |= flagdesc[i].flag;
+				if (flagset)
+					result |= flagdesc[i].flag;
 				break;
 			}
 		}
 	}
-	
+
+	if (flagmask)
+		*flagmask = mask;
+
 	return result;
 }
 
-std::string writeFlagString(u32 flags, FlagDesc *flagdesc) {
+std::string writeFlagString(u32 flags, FlagDesc *flagdesc, u32 flagmask)
+{
 	std::string result;
-	
+
 	for (int i = 0; flagdesc[i].name; i++) {
-		if (flags & flagdesc[i].flag) {
+		if (flagmask & flagdesc[i].flag) {
+			if (!(flags & flagdesc[i].flag))
+				result += "no";
+
 			result += flagdesc[i].name;
 			result += ", ";
 		}
 	}
-	
+
 	size_t len = result.length();
 	if (len >= 2)
 		result.erase(len - 2, 2);
-	
+
 	return result;
 }
 
-char *mystrtok_r(char *s, const char *sep, char **lasts) {
+size_t mystrlcpy(char *dst, const char *src, size_t size)
+{
+	size_t srclen  = strlen(src) + 1;
+	size_t copylen = MYMIN(srclen, size);
+
+	if (copylen > 0) {
+		memcpy(dst, src, copylen);
+		dst[copylen - 1] = '\0';
+	}
+
+	return srclen;
+}
+
+char *mystrtok_r(char *s, const char *sep, char **lasts)
+{
 	char *t;
 
 	if (!s)
@@ -172,7 +285,8 @@ char *mystrtok_r(char *s, const char *sep, char **lasts) {
 	return s;
 }
 
-u64 read_seed(const char *str) {
+u64 read_seed(const char *str)
+{
 	char *endptr;
 	u64 num;
 	
